@@ -478,13 +478,15 @@ pub fn index_elim_identity(index: &Index) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn index_fuse(index: &Index) -> Option<Index> {
-    index.node.as_index().map(|inner| {
-        Index::try_new(
-            inner.node.clone(),
-            compose(&index.index, &inner.index),
-            index.name_cloned(),
+    index.node.as_index().and_then(|inner| {
+        Some(
+            Index::try_new(
+                inner.node.clone(),
+                compose(&index.index, &inner.index)?,
+                index.name_cloned(),
+            )
+            .unwrap(),
         )
-        .unwrap()
     })
 }
 
@@ -1532,17 +1534,22 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                     .iter()
                     .filter(|args| {
                         let (child, _axes) = &**args;
-                        child.as_scalar_constant().map_or(true, |x| {
-                            if x.info().shape.len() == 0 {
-                                scalar *= x.value;
-                                reduced_any += 1;
-                                if new_name.is_none() {
+                        if let Some(x) = child.as_scalar_constant() && x.info().shape.len() == 0  {
+                            scalar *= x.value;
+                            reduced_any += 1;
+                            if x.name().is_some() {
+                                if let Some(new_name_string) = &new_name {
+                                    new_name = Some(
+                                        new_name_string.clone() + " + " + x.name().unwrap(),
+                                    );
+                                } else {
                                     new_name = x.name_cloned();
                                 }
-                                return false;
                             }
+                            false
+                        }else{
                             true
-                        })
+                        }
                     })
                     .map(|x| x.clone())
                     .collect();
