@@ -15,6 +15,7 @@ pub use computational_node::{
 };
 pub use constant::{ArrayConstant, ScalarConstant, Symbol};
 pub use generalfunction::{GeneralFunction, GeneralFunctionSpec, PyGFSpecShapeGetter};
+pub use module_nodes::{ModuleNode, ModuleNodeArgSpec, ModuleNodeSpec};
 use num_bigint::BigUint;
 use py_circuit_items::circuit_rust_to_py;
 use pyo3::{exceptions, pyclass::CompareOp};
@@ -45,6 +46,7 @@ pub mod debugging;
 pub mod deep_rewrite;
 pub mod diag_rewrite;
 pub mod generalfunction;
+pub mod module_nodes;
 pub mod named_axes;
 pub mod nrc;
 pub mod print;
@@ -519,6 +521,7 @@ define_circuit!(
     GeneralFunction,
     Concat,
     Scatter,
+    ModuleNode,
 );
 
 /// Define adhoc unions of different circuit types
@@ -839,6 +842,48 @@ pub enum CircuitConstructionError {
     #[error("Unknown GeneralFunction name {spec_name}")]
     UnknownGeneralFunction { spec_name: String },
 
+    #[error("ModuleNode wrong number of children, expected {expected} got {got}")]
+    ModuleNodeWrongNumberChildren { expected: usize, got: usize },
+
+    #[error("ModuleNode incompatible shapes, got {got:?} default {default:?}")]
+    ModuleNodeIncompatibleShapes {
+        got: Vec<Shape>,
+        default: Vec<Shape>,
+    },
+
+    #[error("ModuleNode expansion error {error}")]
+    ModuleNodeExpansionError {
+        error: Box<CircuitConstructionError>,
+    },
+
+    #[error("Batching Rank Too Low")]
+    BatchingRankTooLow {
+        default: Vec<usize>,
+        got: Vec<usize>,
+    },
+
+    #[error("Trying to expand fixed index, index {index:?} old shape{old_shape:?} new shape {new_shape:?}")]
+    ExpandingFixedIndex {
+        index: TensorIndex,
+        old_shape: Shape,
+        new_shape: Shape,
+    },
+
+    #[error(
+        "Trying to expand concat axis, index {axis} old shape{old_shape:?} new shape {new_shape:?}"
+    )]
+    ExpandingConcatAxis {
+        axis: usize,
+        old_shape: Shape,
+        new_shape: Shape,
+    },
+
+    #[error("Inputs that should have same batching have different batchings, {batch_shapes:?}")]
+    InconsistentBatches { batch_shapes: Vec<Shape> },
+
+    #[error("ModuleNode got unknown keyword argument, {argument}")]
+    ModuleNodeUnknownArgument { argument: String },
+
     #[error("python error {py_err:?}")]
     PythonError { py_err: Rc<PyErr> },
 }
@@ -857,6 +902,8 @@ impl From<PyErr> for CircuitConstructionError {
     }
 }
 
+pub type CircResult = Result<CircuitRc, CircuitConstructionError>;
+
 #[derive(Error, Debug, Clone)]
 pub enum TensorEvalError {
     #[error("not explicitly computable: {circuit:?})")]
@@ -1092,6 +1139,24 @@ where
     recurse(circuit)
 }
 
+pub fn deep_map_fallible_pre_new_children<F>(
+    circuit: &Circuit,
+    f: F,
+) -> Result<CircuitRc, CircuitConstructionError>
+where
+    F: Fn(&Circuit, &Vec<CircuitRc>) -> Result<CircuitRc, CircuitConstructionError>,
+{
+    #[apply(cached_lambda)]
+    #[key(circ.info().hash, HashBytes)]
+    fn recurse(circ: &Circuit) -> Result<CircuitRc, CircuitConstructionError> {
+        let old_children: Vec<CircuitRc> = circ.children().collect();
+        let new_children: Result<Vec<CircuitRc>, CircuitConstructionError> =
+            old_children.iter().map(|x| &***x).map(recurse).collect();
+        new_children.and_then(|a| f(circ, &a))
+    }
+    recurse(circuit)
+}
+
 pub fn apply_fn_cache<I, K, O, F, FK>(i: &I, f: F, c: &mut HashMap<K, O>, fk: FK) -> O
 where
     F: Fn(&I) -> O,
