@@ -310,7 +310,7 @@ pub fn evaluate_schedule_adjust_numerical_scale(
         })
     };
     let clamp = |tup: &(Tensor, f64)| -> (Tensor, f64) {
-        let scale: f64 = tensor_scale(&tup.0).unwrap();
+        let scale = tensor_scale(&tup.0).unwrap();
         if (scale > settings.numerical_scale_max || scale < settings.numerical_scale_min)
             && scale != 0.0
         {
