@@ -313,3 +313,10 @@ pub fn hashmap_collect_except_duplicates<K: Eq + Hash + Clone, V: Eq + Hash + Cl
 
 pub type BitMask64 = usize;
 pub type BitMask128 = u128;
+
+#[macro_export]
+macro_rules! ss {
+    ($s:literal) => {
+        Some($s.to_owned())
+    };
+}
